/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.extractors;

import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.ELog;
import it.eng.spago4q.bo.EOperation;
import it.eng.spago4q.bo.EProcess;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QFKDAO;
import it.eng.spago4q.extractors.bo.GenericItemInterface;
import it.eng.spago4q.extractors.bo.InterfaceItem;
import it.eng.spago4q.extractors.script.ScriptExecutor;
import it.eng.spago4q.extractors.utilities.OperationUtil;

import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

public class DefaultCoordinator implements OperationCoordinatorInterface {

	private List operationsList = null;
	private EProcess eProcess;
	private static final String EOPERATION_DAO = "EOperationDAO";

	private static Logger logger = Logger.getLogger(DefaultCoordinator.class);

	public void init(final EProcess eProcessPar) throws EMFUserError {
		this.eProcess = eProcessPar;
		IES4QFKDAO eOperationDAO = (IES4QFKDAO) DAOFactory
				.getDAO(EOPERATION_DAO);
		this.operationsList = eOperationDAO.loadObjectList(eProcessPar.getId(),
				null, null);
	}

	public List getOperationsList() {
		return operationsList;
	}

	public void run() throws EMFUserError {
		ScriptExecutor scriptExecutor = new ScriptExecutor();
		List currentOperationsList = getOperationsList();
		// get the extraction date
		Date executionDate = Calendar.getInstance().getTime();
		for (Iterator iterator = currentOperationsList.iterator(); iterator.hasNext();) {

			EOperation operation = (EOperation) iterator.next();

			DomainValueCacheCreator domainValueCacheCreator = new DomainValueCacheCreator(
					operation.getInterfaceType().getId());

			List<GenericItemInterface> genericItemList = OperationUtil
					.executeOperation(operation);

			ELog eLog = new ELog();
			eLog.setProcessId(eProcess.getId());
			eLog.setOperationId(operation.getId());
			eLog.setExtracted(genericItemList.size());
			eLog.setOperationDate(executionDate);
			eLog.setExecutionDate(executionDate);

			IES4QFKDAO eLogDao = (IES4QFKDAO) DAOFactory.getDAO("ELogDAO");

			Integer logId = eLogDao.insertObject(eLog);

			eLog.setIdLog(logId);

			List<InterfaceItem> interfaceItem = MapMaker.mapping(operation,
					genericItemList, scriptExecutor, domainValueCacheCreator);

			FactTableManager factTableManager = new FactTableManager(operation
					.getInterfaceType().getId());
			int numberOfInsert = 0;

			for (Iterator iterator2 = interfaceItem.iterator(); iterator2
					.hasNext();) {
				InterfaceItem interfaceItem2 = (InterfaceItem) iterator2.next();
				if (factTableManager.loadData(executionDate, operation.getInterfaceType()
						.getId(), interfaceItem2, logId)) {
					numberOfInsert++;
				}
			}

			eLog.setInserted(numberOfInsert);
			eLogDao.modifyObject(eLog);
		}
	}

}
