/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.extractors;

import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.EOperation;
import it.eng.spago4q.bo.EOperationField;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QFKDAO;
import it.eng.spago4q.extractors.bo.GenericItemInterface;
import it.eng.spago4q.extractors.bo.InterfaceFieldInstance;
import it.eng.spago4q.extractors.bo.InterfaceItem;
import it.eng.spago4q.extractors.script.ScriptExecutor;
import it.eng.spago4q.extractors.utilities.ExtractorConstants;
import org.apache.commons.lang.StringEscapeUtils;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public final class MapMaker {
	
	private MapMaker(){}

	public static  List<InterfaceItem> mapping(final EOperation operation,
			final List<GenericItemInterface> genericItemList,
			final ScriptExecutor scriptExecutor,
			final DomainValueCacheCreator domainValueCache) throws EMFUserError {
		
		List<InterfaceItem> toReturn = new ArrayList<InterfaceItem>();

		IES4QFKDAO eOperationFieldDao = (IES4QFKDAO) DAOFactory
				.getDAO("EOperationFieldDAO");

		List operationFieldList = eOperationFieldDao.loadObjectList(operation
				.getId(), null, null);

		for (Iterator<GenericItemInterface> iterator = genericItemList
				.iterator(); iterator.hasNext();) {
			InterfaceItem interfaceItem = new InterfaceItem();
			GenericItemInterface genericItem = iterator.next();
			for (Iterator iterator2 = operationFieldList.iterator(); iterator2
					.hasNext();) {
				EOperationField operationField = (EOperationField) iterator2
						.next();

				// get value from generic Item
				String value = genericItem.getValue(operationField.getName());
				
				
				//tansform value with script
				if(operationField.getScript()!= null && operationField.getScript().getId()!= null){
					value = scriptExecutor.execute(operationField.getScript().getId(),
							operationField.getScript().getScriptType(),
							value);
				}
				
				Object objectValue = transform(value, operationField
						.getInterfaceField().getFieldType(), domainValueCache,
						operationField.getInterfaceField().getName());
				//

				InterfaceFieldInstance interfaceFieldInstance = new InterfaceFieldInstance(
						objectValue, operationField.getInterfaceField()
								.getSensible(), operationField
								.getInterfaceField().getKeyField(),
						operationField.getInterfaceField().getFieldType());
				interfaceItem.setValue(operationField.getInterfaceField()
						.getName(), interfaceFieldInstance);
			}
			if (operationFieldList.size()!= 0){
				toReturn.add(interfaceItem);
			}
		}
		
		return toReturn;

	}

	private static  Object transform(final String value, final String type,
			final DomainValueCacheCreator domainValueCache, final String name) {
		Object toReturn = null;
		if (!domainValueCache.hasDomainValue(name)) {
			
			if (type != null && type.equalsIgnoreCase(ExtractorConstants.STRING) && value != null) {
				toReturn = StringEscapeUtils.escapeSql(value);
			}
			if (type != null && type.equalsIgnoreCase(ExtractorConstants.INTEGER) && value != null) {
				try{	
					toReturn = Integer.parseInt(value);
				} catch(NumberFormatException e){
					// TODO add logger
					toReturn = null;
				}
			}
			if (type != null && type.equalsIgnoreCase(ExtractorConstants.DOUBLE) && value != null) {
				try{
					toReturn = new Double(value);
				} catch(NumberFormatException e){
					// TODO add logger
					toReturn = null;
				}
			}
			if (type != null && type.equalsIgnoreCase(ExtractorConstants.BOOLEAN)) {
				toReturn = value;
			}
			if (type != null && type.equalsIgnoreCase(ExtractorConstants.DATE)) {
				toReturn = value;
			}
		} else {
			toReturn = domainValueCache.getDomainValueId(name, value);
		}
		return toReturn;
	}

}
