/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.extractors.script;

import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.extractors.utilities.ExtractorConstants;

public class ScriptExecutor {

	private ScriptCache groovyCache;
	private DateConverterCache dateConverterCache;
	
	public ScriptExecutor(){
		groovyCache = new ScriptCache();
		dateConverterCache = new DateConverterCache();
	}
	
	public String execute(final Integer scriptId, final String scriptType, final String value) {
		String toReturn = "";
		if (scriptType != null && scriptType.equals(ExtractorConstants.GROOVY)){
			toReturn = executeGroovy(scriptId, value);
		}
		if (scriptType != null && scriptType.equals(ExtractorConstants.DATE_CONVERTER)) {
			toReturn = executeDateConverter(scriptId, value);
		}
		return toReturn;
	}	

	private ScriptCache getGroovyCache(){
		return groovyCache;
	}
	
	private DateConverterCache getDateConverterCache(){
		return dateConverterCache;
	}
	
	private String executeGroovy(final Integer scriptId, final String value){
		String toReturn = "";
		IMappingScript script = getGroovyCache().getScript(scriptId);
		if (script!= null){
			toReturn = script.execute(value);
		}else {
			toReturn = value;
		}
		return toReturn;
	}
	
	private String executeDateConverter(final Integer scriptId, final String value){
		String toReturn = null;
		String format = getDateConverterCache().getDateFormat(scriptId);
		try {
			toReturn = DateConverterScript.execute(value, format);
		} catch (EMFUserError e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return toReturn; 
	}

}
