/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.extractors.tablehandlers.postgresql;

import it.eng.spago.dbaccess.SQLStatements;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.InterfaceField;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QFKDAO;
import it.eng.spago4q.extractors.tablehandlers.AbstractSqlCodeGenerator;
import it.eng.spago4q.extractors.utilities.ExtractorConstants;

import java.util.Iterator;
import java.util.List;

public class PostgreSqlCodeGenerator extends AbstractSqlCodeGenerator {
	
	private static String POSTGRESQL = "POSTGRESQL";

	private static final String SEQUENCE_NAME_PLACE_HOLDER = "@sequenceName";
	private static final String CONSTRAINT_NAME_PLACE_HOLDER = "@constraintName";


	private static String createSequenceName(final String tableName) {
		return tableName + "_SEQ";
	}

	private static String createPrimaryKeyConstraintName(final String tableName) {
		return "XPK" + tableName;
	}

	public static String generateCreateSequenceQuery(final String tableName) {

		String query = SQLStatements
				.getStatement("POSTGRE_CREATE_TABLE_SEQUENCE");
		query = query.replace(SEQUENCE_NAME_PLACE_HOLDER,
				createSequenceName(tableName));
		return query;
	}

	public String generateCreateTableQuery(final String tableName,
			final List interfaceFieldList) throws EMFUserError {
		String columnNameType = "";
		for (Iterator iterator = interfaceFieldList.iterator(); iterator
				.hasNext();) {
			InterfaceField interfaceField = (InterfaceField) iterator.next();

			if (!isValidColumName(interfaceField.getName())) {
				throw new EMFUserError(EMFErrorSeverity.ERROR, 10035);
			}

			columnNameType += ", " + interfaceField.getName() + " ";
			IES4QFKDAO eDomainValueDAO = (IES4QFKDAO) DAOFactory
					.getDAO("EDomainValueDAO");
			List domainValueList = eDomainValueDAO.loadObjectList(
					interfaceField.getId(), null, null);

			if (domainValueList == null
					|| (domainValueList.isEmpty())) {
				columnNameType += typeConverter(interfaceField.getFieldType());
			} else {
				columnNameType += typeConverter(ExtractorConstants.INTEGER);
			}
		}

		String commandString = SQLStatements
				.getStatement("POSTGRE_CREATE_TABLE");
		commandString = commandString.replace(TABLE_NAME_PLACE_HOLDER,
				tableName);
		commandString = commandString.replace(COLUMN_NAME_TYPE_PLACE_HOLDER,
				columnNameType);
		commandString = commandString.replace(CONSTRAINT_NAME_PLACE_HOLDER,
				createPrimaryKeyConstraintName(tableName));

		commandString = commandString.replace(SEQUENCE_NAME_PLACE_HOLDER,
				createSequenceName(tableName));

		// logger.debug("Return this Create query:" + commandString);
		System.out.println("Return this Create query:" + commandString);
		return commandString;
	}

	public static String generateDropSequenceQuery(final String tableName) {
		String query = SQLStatements
				.getStatement("POSTGRE_DROP_TABLE_SEQUENCE");
		query = query.replace(SEQUENCE_NAME_PLACE_HOLDER,
				createSequenceName(tableName));
		return query;
	}

	@Override
	public String getDBConfEnvelopeName() {
		return POSTGRESQL;
	}

}
