/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago4q.source.utils;

import it.eng.spago.error.EMFUserError;
import it.eng.spago4q.bo.DataSource;
import it.eng.spago4q.bo.EOperation;
import it.eng.spago4q.dao.DAOFactory;
import it.eng.spago4q.dao.IES4QFKDAO;
import it.eng.spago4q.extractors.ExtractorInterface;

public class SourceTypeUtil {
	
	private SourceTypeUtil() {}

	public static TestMessage testDataSource(final Integer idDataSource) {
		String message;
		boolean isOk;

		IES4QFKDAO dataSourceDAO;
		try {
			dataSourceDAO = (IES4QFKDAO) DAOFactory.getDAO("EDataSourceDAO");
			DataSource dataSourceBo = (DataSource) dataSourceDAO
			.loadObjectById(idDataSource);
			String extractorClassName = dataSourceBo.getSourceType()
			.getExtractorClass();
			ExtractorInterface extractor = (ExtractorInterface) Class.forName(
					extractorClassName).newInstance();
			message = extractor.dataSourceTest(idDataSource);
			isOk = true;
		} catch (EMFUserError e) {
			message = e.getDescription();
			isOk = false;
		} catch (InstantiationException e) {
			message = "Extractor class problem.<br>" + e.toString();
			isOk = false;
		} catch (IllegalAccessException e) {
			message = "Extractor class problem.<br>" + e.toString();
			isOk = false;
		} catch (ClassNotFoundException e) {
			message = "Extractor class problem.<br>" + e.toString();
			isOk = false;
		}
		TestMessage toReturn = new TestMessage(isOk,message);
		return toReturn;
	}

	public static TestMessage testOperation(final Integer idOperation) {
		String message;
		boolean isOk;

		IES4QFKDAO operationDAO;

		try {
			operationDAO = (IES4QFKDAO) DAOFactory.getDAO("EOperationDAO");
			EOperation operationBo = (EOperation) operationDAO
			.loadObjectById(idOperation);
			String extractorClassName = operationBo.getDataSource().getSourceType()
			.getExtractorClass();
			ExtractorInterface extractor = (ExtractorInterface) Class.forName(
					extractorClassName).newInstance();
			message = extractor.operationTest(idOperation);
			isOk = true;
		} catch (EMFUserError e) {
			message = e.getDescription();
			isOk = false;
		} catch (InstantiationException e) {
			message = "Extractor class problem.<br>" + e.toString();
			isOk = false;
		} catch (IllegalAccessException e) {
			message = "Extractor class problem.<br>" + e.toString();
			isOk = false;
		} catch (ClassNotFoundException e) {
			message = "Extractor class problem.<br>" + e.toString();
			isOk = false;
		}
		TestMessage toReturn = new TestMessage(isOk,message);
		return toReturn;
	}

}
